import json
import math
import os
from datetime import datetime
from functools import cached_property
from time import sleep

import boto3
from botocore.exceptions import ClientError

from reach_commons.app_logging.logger import get_reach_logger
from reach_commons.reach_aws.exceptions import KinesisClientException
from reach_commons.utils import ReachChunkObjectsUtils, remove_nulls


# noinspection PyMethodMayBeStatic
class BaseFirehoseClient:
    def __init__(
        self,
        logger=get_reach_logger(),
        region_name="us-east-1",
        profile_name=None,
    ):
        self.region_name = region_name
        self.profile_name = profile_name
        self.logger = logger

    def handle_exception(self, exc, secret, ciphertext):
        error_msg = (
            "error_kinesis_client, "
            "secret={}, "
            "ciphertext={!r}, "
            "error={!r}".format(secret, ciphertext, exc)
        )
        self.logger.error(error_msg)
        raise KinesisClientException(error_msg)


class FirehoseClient(BaseFirehoseClient):
    def __init__(self, delivery_stream_name=None):
        super().__init__()
        self.delivery_stream_name = (
            delivery_stream_name
            or f"{os.environ.get('ENV', 'Staging')}-kinesis-firehose-extended-s3-stream"
        )

    @cached_property
    def client(self):
        session = boto3.Session(
            region_name=self.region_name, profile_name=self.profile_name
        )
        return session.client("firehose")

    def notify_google_review_downloaded(
        self,
        business_id: int,
        partner_name: str,
        object_payload,
        method_name: str,
        split_array_path=None,
        retry: int = 0,
    ):
        object_payload = (
            remove_nulls(object_payload)
            if isinstance(object_payload, dict)
            else object_payload
        )

        def attempt_send(payload, current_retry):
            firehose_message = {
                "DateTime": str(datetime.now()),
                "MinuteGroup": int(15 * (math.floor(datetime.now().minute / 15))),
                "BusinessID": business_id,
                "PartnerName": partner_name,
                "ObjectName": "GoogleReviewDownloaded",
                "MethodName": method_name,
                "ObjectPayload": payload,
            }
            try:
                record = {"Data": json.dumps(firehose_message)}
                response = self.client.put_record(
                    DeliveryStreamName=self.delivery_stream_name, Record=record
                )
                self.logger.info(
                    f"Successfully sent data to Firehose for BusinessID: {business_id}, Retry attempt: {current_retry}"
                )

                return response
            except Exception as ex:
                self.logger.exception(
                    "An error occurred while trying to send data to Firehose",
                    exc_info=ex,
                )
                if current_retry < 20:
                    sleep(1)
                    self.logger.info(
                        f"Retrying to send data to Firehose for BusinessID: {business_id}, "
                        f"Next retry attempt: {current_retry + 1}"
                    )
                    return attempt_send(payload, current_retry + 1)

        if split_array_path:
            self.logger.info(
                f"Splitting payload for BusinessID: {business_id} using path: {split_array_path}"
            )

            chunked_payloads = ReachChunkObjectsUtils.chunk_list_items(
                obj_to_split=object_payload, path_of_array=split_array_path
            )

            for chunk in chunked_payloads:
                self.logger.info(
                    f"Sending a chunked payload part for BusinessID: {business_id}"
                )

                attempt_send(chunk, retry)
        else:
            self.logger.info(
                f"Sending payload for BusinessID: {business_id} without splitting"
            )

            return attempt_send(object_payload, retry)



import os

import requests


class EventProcessorClient:
    def __init__(self):
        self.environment = os.environ.get("ENV", "Staging")

    @property
    def base_url(self):
        return {
            "Staging": "https://90tin0ndr1.execute-api.us-east-1.amazonaws.com",
            "Prod": "https://101iu6hpaj.execute-api.us-east-1.amazonaws.com",
        }.get(self.environment)

    @property
    def gateway_headers(self):
        common_headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }

        auth_token = {
            "Staging": "Bearer 6c6adc18-22a9-929f-648d-786eb20ebcf4",
            "Prod": "Bearer a57a82c0-0493-563b-ba1d-6c63c201ce20",
        }.get(self.environment)

        headers = {**common_headers, "Authorization": auth_token}

        return headers

    def review_profile_patch_business(self, business_id):
        headers = {**self.gateway_headers, "X-Api-Version": "v2"}
        resp = requests.patch(
            f"{self.base_url}/v1/review-profile/business/{business_id}", headers=headers
        )
        return resp

    def review_profile_post_business(self, business_id):
        headers = {**self.gateway_headers, "X-Api-Version": "v2"}
        resp = requests.post(
            f"{self.base_url}/v1/review-profile/business/{business_id}",
            headers=headers,
        )
        return resp

    def review_profile_delete_business(self, business_id):
        headers = {**self.gateway_headers, "X-Api-Version": "v2"}
        resp = requests.delete(
            f"{self.base_url}/v1/review-profile/business/{business_id}",
            headers=headers,
        )
        return resp

    def create_twilio_phone_number_by_business(self, business_id):
        headers = {**self.gateway_headers}
        resp = requests.post(
            f"{self.base_url}/v1/business/{business_id}/twillio",
            headers=headers,
        )
        return resp



import os

import requests


class OutscraperClient:
    def __init__(self):
        self.environment = os.environ.get("ENV", "Staging")
        self.base_url = "https://api.app.outscraper.com"

    @property
    def headers(self):
        common_headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }

        auth_token = {
            "Staging": "MjZjYWUyMzg4MzUxNDk2Zjk0OGRlOTFlMzkyMDhhYWF8ZDhlNzM5OGVmOQ",
            "Prod": "MjZjYWUyMzg4MzUxNDk2Zjk0OGRlOTFlMzkyMDhhYWF8ZDhlNzM5OGVmOQ",
        }.get(self.environment)

        return {**common_headers, "X-API-KEY": auth_token}

    @property
    def webhook_url(self):
        return {
            "Staging": "https://api-staging.getreach.ai/webhooks/data-bridge/outscraper/scraper-completed?token=tRm4k75N9hK1vF0dSwIi0HcMyV",
            "Prod": "https://api.getreach.ai/webhooks/data-bridge/outscraper/scraper-completed?token=tRm4k75N9hK1vF0dSwIi0HcMyV",
        }.get(self.environment)

    def search_google_maps_reviews(self, google_place_id: str, start_from: int):
        params = {
            "query": google_place_id,
            "webhook": self.webhook_url,
            "reviewsLimit": 0,
        }
        if start_from:
            params["cutoff"] = start_from
        resp = requests.get(
            f"{self.base_url}/maps/reviews-v3",
            headers=self.headers,
            params=params,
        )
        return resp

    def get_job_details(self, job_id: str):
        """Perform a GET request to retrieve details for a specific job using its job_id."""
        url = f"{self.base_url}/requests/{job_id}"
        response = requests.get(url, headers=self.headers)
        return response



import json
import logging
import os


class Singleton(type):
    """
    Define an Instance operation that lets clients access its unique
    instance.
    """

    def __init__(cls, name, bases, attrs, **kwargs):
        super().__init__(name, bases, attrs)
        cls._instance = None

    def __call__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__call__(*args, **kwargs)
        return cls._instance


class ReachLogger(metaclass=Singleton):
    logger = logging.getLogger()
    logger.setLevel(os.environ.get("logger_level") or logging.INFO)
    context = {"logger_identifier": "reach_logger"}

    def set_context(self, **kwargs):
        self.context = {"logger_identifier": "reach_logger", **kwargs}

    # noinspection PyBroadException
    def _log(self, level, msg, *args, **kwargs):
        log_message = {**self.context, "level": logging.getLevelName(level)}

        if isinstance(msg, dict):
            log_message.update(msg)
        else:
            log_message["message"] = str(msg)

        if args and isinstance(args[0], dict):
            log_message.update(args[0])
        try:
            log_message_json = json.dumps(log_message)
        except Exception:
            log_message_json = str(log_message)

        self.logger.log(level, log_message_json, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        self._log(logging.INFO, msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        self._log(logging.ERROR, msg, *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        self._log(logging.DEBUG, msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        self._log(logging.WARNING, msg, *args, **kwargs)

    def exception(self, msg, *args, **kwargs):
        self._log(logging.ERROR, msg, *args, **kwargs)

    # noinspection PyPep8Naming
    def setLevel(self, level: str):
        self.logger.setLevel(level=level or logging.INFO)


logger = ReachLogger()


def set_business_id(business_id, pos_partner, level=logging.INFO):
    logger.set_context(business=business_id, partner=pos_partner)
    logger.setLevel(level)


def get_reach_logger():
    return logger
